@@ -46,10 +46,11 @@ extern "C" {
 #endif
 
 /* To easily un-inline functions for debugging */
-#ifndef	INLINE
-#define INLINE inline
+#ifndef GS_STATIC_INLINE
+#define GS_STATIC_INLINE static inline
 #endif
 
+
 /*
  *      NB. This file is intended for internal use by the GNUstep libraries
  *      and may change siugnificantly between releases.
@@ -374,6 +375,11 @@ struct	_GSIMapTable {
   GSI_MAP_EXTRA	extra;
 #endif
 };
+#define GSI_MAP_TABLE_T GSIMapTable_t
+#endif
+
+#ifndef GSI_MAP_TABLE_S
+#define GSI_MAP_TABLE_S sizeof(GSI_MAP_TABLE_T)
 #endif
 
 typedef struct	_GSIMapEnumerator {
@@ -389,27 +395,27 @@ typedef struct _GSIMapEnumerator GSIMapEnumerator_t;
 #endif
 typedef GSIMapEnumerator_t	*GSIMapEnumerator;
 
-static INLINE GSIMapBucket
+GS_STATIC_INLINE GSIMapBucket
 GSIMapPickBucket(unsigned hash, GSIMapBucket buckets, uintptr_t bucketCount)
 {
   return buckets + hash % bucketCount;
 }
 
-static INLINE GSIMapBucket
+GS_STATIC_INLINE GSIMapBucket
 GSIMapBucketForKey(GSIMapTable map, GSIMapKey key)
 {
   return GSIMapPickBucket(GSI_MAP_HASH(map, key),
     map->buckets, map->bucketCount);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapLinkNodeIntoBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   node->nextInBucket = bucket->firstNode;
   bucket->firstNode = node;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapUnlinkNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   if (node == bucket->firstNode)
@@ -429,14 +435,14 @@ GSIMapUnlinkNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
   node->nextInBucket = 0;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapAddNodeToBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   GSIMapLinkNodeIntoBucket(bucket, node);
   bucket->nodeCount += 1;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapAddNodeToMap(GSIMapTable map, GSIMapNode node)
 {
   GSIMapBucket	bucket;
@@ -446,21 +452,21 @@ GSIMapAddNodeToMap(GSIMapTable map, GSIMapNode node)
   map->nodeCount++;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveNodeFromBucket(GSIMapBucket bucket, GSIMapNode node)
 {
   bucket->nodeCount--;
   GSIMapUnlinkNodeFromBucket(bucket, node);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveNodeFromMap(GSIMapTable map, GSIMapBucket bkt, GSIMapNode node)
 {
   map->nodeCount--;
   GSIMapRemoveNodeFromBucket(bkt, node);
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
 {
   GSI_MAP_RELEASE_KEY(map, node->key);
@@ -474,7 +480,7 @@ GSIMapFreeNode(GSIMapTable map, GSIMapNode node)
   map->freeNodes = node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapRemoveAndFreeNode(GSIMapTable map, uintptr_t bkt, GSIMapNode node)
 {
   GSIMapNode	next = node->nextInBucket;
@@ -483,7 +489,7 @@ GSIMapRemoveAndFreeNode(GSIMapTable map, uintptr_t bkt, GSIMapNode node)
   return next;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemoveWeak(GSIMapTable map)
 {
   if (GSI_MAP_ZEROED(map))
@@ -511,7 +517,7 @@ GSIMapRemoveWeak(GSIMapTable map)
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRemangleBuckets(GSIMapTable map,
   GSIMapBucket old_buckets, uintptr_t old_bucketCount,
   GSIMapBucket new_buckets, uintptr_t new_bucketCount)
@@ -560,7 +566,7 @@ GSIMapRemangleBuckets(GSIMapTable map,
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapMoreNodes(GSIMapTable map, unsigned required)
 {
   GSIMapNode	*newArray;
@@ -631,7 +637,7 @@ GSIMapMoreNodes(GSIMapTable map, unsigned required)
     }
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
 {
   GSIMapNode	node = bucket->firstNode;
@@ -660,7 +666,7 @@ GSIMapNodeForKeyInBucket(GSIMapTable map, GSIMapBucket bucket, GSIMapKey key)
   return node;
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket;
@@ -675,7 +681,7 @@ GSIMapNodeForKey(GSIMapTable map, GSIMapKey key)
   return node;
 }
 
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapFirstNode(GSIMapTable map)
 {
   if (map->nodeCount > 0)
@@ -724,7 +730,7 @@ GSIMapFirstNode(GSIMapTable map)
  * or pointer values that are their own hash values (when converted to unsigned
  * integers) and can be compared with a test for integer equality.
  */
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket;
@@ -759,7 +765,7 @@ GSIMapNodeForSimpleKey(GSIMapTable map, GSIMapKey key)
 }
 #endif
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapResize(GSIMapTable map, uintptr_t new_capacity)
 {
   GSIMapBucket	new_buckets;
@@ -815,7 +821,7 @@ GSIMapResize(GSIMapTable map, uintptr_t new_capacity)
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapRightSizeMap(GSIMapTable map, uintptr_t capacity)
 {
   /* FIXME: Now, this is a guess, based solely on my intuition.  If anyone
@@ -849,7 +855,7 @@ GSIMapRightSizeMap(GSIMapTable map, uintptr_t capacity)
  * is in progress.  The results of doing so are reasonably unpredictable.
  * <br />Remember, DON'T MESS WITH A MAP WHILE YOU'RE ENUMERATING IT.
  */
-static INLINE GSIMapEnumerator_t
+GS_STATIC_INLINE GSIMapEnumerator_t
 GSIMapEnumeratorForMap(GSIMapTable map)
 {
   GSIMapEnumerator_t	enumerator;
@@ -893,7 +899,7 @@ GSIMapEnumeratorForMap(GSIMapTable map)
 /**
  * Tidies up after map enumeration ... effectively destroys the enumerator.
  */
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapEndEnumerator(GSIMapEnumerator enumerator)
 {
   ((_GSIE)enumerator)->map = 0;
@@ -907,7 +913,7 @@ GSIMapEndEnumerator(GSIMapEnumerator enumerator)
  * bucket and node to remove the node from the map using the
  * GSIMapRemoveNodeFromMap() function.
  */
-static INLINE GSIMapBucket 
+GS_STATIC_INLINE GSIMapBucket 
 GSIMapEnumeratorBucket(GSIMapEnumerator enumerator)
 {
   if (((_GSIE)enumerator)->node != 0)
@@ -922,7 +928,7 @@ GSIMapEnumeratorBucket(GSIMapEnumerator enumerator)
 /**
  * Returns the next node in the map, or a nul pointer if at the end.
  */
-static INLINE GSIMapNode 
+GS_STATIC_INLINE GSIMapNode 
 GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
 {
   GSIMapNode	node = ((_GSIE)enumerator)->node;
@@ -1000,7 +1006,7 @@ GSIMapEnumeratorNextNode(GSIMapEnumerator enumerator)
  * Used to implement fast enumeration methods in classes that use GSIMap for
  * their data storage.
  */
-static INLINE NSUInteger 
+GS_STATIC_INLINE NSUInteger 
 GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
                                               NSFastEnumerationState *state,
                                               id *stackbuf,
@@ -1057,7 +1063,7 @@ GSIMapCountByEnumeratingWithStateObjectsCount(GSIMapTable map,
 }
 
 #if	GSI_MAP_HAS_VALUE
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1076,7 +1082,7 @@ GSIMapAddPairNoRetain(GSIMapTable map, GSIMapKey key, GSIMapVal value)
   return node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1097,7 +1103,7 @@ GSIMapAddPair(GSIMapTable map, GSIMapKey key, GSIMapVal value)
   return node;
 }
 #else
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1115,7 +1121,7 @@ GSIMapAddKeyNoRetain(GSIMapTable map, GSIMapKey key)
   return node;
 }
 
-static INLINE GSIMapNode
+GS_STATIC_INLINE GSIMapNode
 GSIMapAddKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapNode	node = map->freeNodes;
@@ -1139,7 +1145,7 @@ GSIMapAddKey(GSIMapTable map, GSIMapKey key)
  * Removes the item for the specified key from the map.
  * If the key was present, returns YES, otherwise returns NO.
  */
-static INLINE BOOL
+GS_STATIC_INLINE BOOL
 GSIMapRemoveKey(GSIMapTable map, GSIMapKey key)
 {
   GSIMapBucket	bucket = GSIMapBucketForKey(map, key);
@@ -1155,7 +1161,7 @@ GSIMapRemoveKey(GSIMapTable map, GSIMapKey key)
   return NO;
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapCleanMap(GSIMapTable map)
 {
   if (map->nodeCount > 0)
@@ -1202,7 +1208,7 @@ GSIMapCleanMap(GSIMapTable map)
     }
 }
 
-static INLINE void
+GS_STATIC_INLINE void
 GSIMapEmptyMap(GSIMapTable map)
 {
 #ifdef	GSI_MAP_NOCLEAN
@@ -1243,7 +1249,7 @@ GSIMapEmptyMap(GSIMapTable map)
   map->zone = 0;
 }
 
-static INLINE void 
+GS_STATIC_INLINE void 
 GSIMapInitWithZoneAndCapacity(GSIMapTable map, NSZone *zone, uintptr_t capacity)
 {
   map->zone = zone;
@@ -1258,6 +1264,37 @@ GSIMapInitWithZoneAndCapacity(GSIMapTable map, NSZone *zone, uintptr_t capacity)
   GSIMapMoreNodes(map, capacity);
 }
 
+GS_STATIC_INLINE NSUInteger 
+GSIMapSize(GSIMapTable map)
+{
+  NSUInteger    index;
+  NSUInteger    size;
+  GSIMapNode	node;
+
+  /* Map table plus arrays of pointers to chunks
+   */
+  size = GSI_MAP_TABLE_S + map->chunkCount * sizeof(void*);
+
+  /* Add the array of buckets.
+   */
+  size += map->bucketCount * sizeof(GSIMapBucket_t);
+
+  /* Add the free nodes.
+   */
+  for (node = map->freeNodes; 0 != node; node = node->nextInBucket)
+    {
+      size += sizeof(GSIMapNode_t);
+    }
+
+  /* Add the used nodes (in the buckets).
+   */
+  for (index = 0; index < map->bucketCount; index++)
+    {
+      size += sizeof(GSIMapNode_t) * map->buckets[index].nodeCount;
+    }
+  return size;
+}
+
 #if	defined(__cplusplus)
 }
 #endif
