@@ -36,10 +36,34 @@ extern "C" {
 
 #if	OS_API_VERSION(GS_API_NONE,GS_API_LATEST)
 
+@interface NSURL (GNUstepBaseAdditions)
+
+/** Builds a URL from components as returned by the methods of the same names.
+ */
+- (id) initWithScheme: (NSString*)scheme
+		 user: (NSString*)user
+	     password: (NSString*)password
+		 host: (NSString*)host
+		 port: (NSNumber*)port
+	     fullPath: (NSString*)fullPath
+      parameterString: (NSString*)parameterString
+		query: (NSString*)query
+	     fragment: (NSString*)fragment;
+
+@end
+
 @interface NSURL (GNUstepBase)
 /** Returns the full path for this URL including any trailing slash.
  */
 - (NSString*) fullPath;
+
+/** Returns the full path for this URL, without decoding percent-escapes.<br />
+ * This is useful if you need to distinguish between "/" and "%2F" in the path.
+ * The normal -path method will convert all "%2F" value in the path into "/"
+ * so that you have no way of knowing whether a "/" is part of a path component
+ * ir is a path separator.
+ */
+- (NSString*) pathWithEscapes;
 @end
 
 #endif	/* OS_API_VERSION */
